% Model1.m
% 
% GEOL 5700 - Sed Trans Mech
% 
% Modeling assignment 1 - Experiments with water volume balance in buckets 
% and basins
% 
% Shahen Huda
% 
% 28 Jan 2012

close all; clear all; clc;

%% Leaky Bucket Numerically (#1)
close all; clear all; clc;

%%%%%% constants

g = 9.81;   %m/s2
R = 0.2;    %m (radius of bucket - taken from RSA code)
rhow = 1000; %kg/m3
A = pi*R^2; %m2 (surface area)

r = 0.01;   %m (radius of hole)
H0 = 0.3;   %m (initial water height)
Qin0 = 0.001; %m3/s

%%%%%% model setup

dt = 1;     %s
tmax = 2000; %s
t = 0:dt:tmax;

H = zeros(size(t)); %preallocate for speed (water height)
H(1) = H0;
Qin = zeros(size(t));

V = H0*A;    %initial volume - constantly changing

% TOGGLE FOR INPUT HISTORIES - 0 = CONST, 1 = OSCILLATING
tog = 1;

if tog == 0
    Qin(1:length(Qin)) = Qin0;
else if tog == 1
        P = 600;    %period
        amp = Qin0/2;   %amplitude of osc
        Qin = Qin0 + amp * sin(2 * pi * t / P);
    end
end

Qout = zeros(size(t));  %preallocate for speed (water out)
% Qout(1) = 0;
u = sqrt(2*H(1)*g);
Qout(1) = u*pi*r^2;

%%%%%% run

for i = 2:length(t)
    u = sqrt(2*H(i-1)*g);   %water out (Torcelli's law)
    Qout(i) = u * pi * r^2;
    
    dVdt = Qin(i) - Qout(i);
    V = V + dVdt * dt;
    V = max(0, V);
    
    H(i) = V/A;
end

figure('units', 'normalized', 'position', [0.3 0.3 0.5 0.6]);
subplot(2,1,1);
plot(t, Qin, 'b');
hold on;
plot(t, Qout, 'r');
ylabel('Q (m^3/s)', 'fontsize', 14);
legend('Q_{in}', 'Q_{out}', 'location', 'southeast');
legend boxoff;
axis([0 max(t) 0 1.1*max(max(Qin), max(Qout))]);
set(gca, 'XTick', 0:max(t)/5:max(t));

subplot(2,1,2);
plot(t, H, 'k');
xlabel('Time (s)', 'fontsize', 14);
ylabel('H (m)', 'fontsize', 14);
axis([0 max(t) 0 1.1*max(H)]);
set(gca, 'XTick', 0:max(t)/5:max(t));


%% Closed Lake Basin (#2)
% close all; 
clear all; clc;

%%%%%% constants
P1 = 0.1;    %m/yr (precip)
RC = 0.5;   %runoff coeff
E = 2.5;    %m/yr (evap)

Ab = 1000 * 1000^2; %m2 (area of the basin)
alpha = 4;  %deg (spread angle)
z0 = 1000;  %m (base elevation)

%%%%%% part b1
zL1 = z0 + tand(alpha) * sqrt(RC * P1 * Ab / (pi * (E + P1 * RC - P1))); %m
disp('2b. Given these conditions, and presuming progression to steady');
disp(['state, zL = ' num2str(zL1, '%.0f') ' m']);
disp(' ');

%%%%%% part b2

P2 = 0:.01:0.8;  %range of P

zL2 = z0 + tand(alpha).* sqrt(RC .* P2 .* Ab ./ (pi * (E + P2.* RC - P2)));

figure;
plot(P2, zL2, 'k');
xlabel('P (m/yr)', 'fontsize', 14);
ylabel('z_L (m)', 'fontsize', 14);
box off;
axis([0 max(P2) z0 100*ceil(max(zL2/100))]);
set(gca, 'XTick', 0:.2:.8);
set(gca, 'YTick', z0:200:(z0 + 10*300));

%%%%%% part c
%test for hysteresis - spin model up from zL1 to zF with 1.93x P
% then let it relax from zF back to zl1 with P.

% filling
zF = 1250;
P3 = 1.9303 * P1;
dt = .1;
tend = 200;
t = 0:dt:tend;

H0 = zL1 - z0;
Hc = zeros(size(t));
Hc(1) = H0;
R = H0 / tand(alpha);
V(1) = 1/3 * pi * H0^3 / ((tand(alpha))^2);
flaggy = 0;

for i = 2:length(t)
    dVdt = P3 * pi * R^2 + RC * P3 * (Ab - pi * R^2) - E * pi * R^2;
    V(i) = V(i-1) + dVdt * dt;
    V(i) = max(0, V(i));
    
    Hc(i) = ((3 * V(i) * (tand(alpha))^2) / pi)^(1/3);
    R = Hc(i) / tand(alpha);
    
    if (Hc(i) > 0.99 * (zF - z0)) && flaggy == 0
        flaggy = i;
    end
    
end

t_99_fill = t(flaggy);
disp('2c. The basin will fill 99% of the difference between current and'); 
disp(['paleo levels in ' num2str(t_99_fill, '%.0f') ' years, given a' ...
    ' 93% increase in P.']);
disp(' ');

% draining
P4 = P1;
dt = 0.1;
tend = 500;
t = 0:dt:tend;
H0 = zF - z0;
Hc2 = zeros(size(t));
Hc2(1) = H0;
R = H0 / tand(alpha);
V(1) = 1/3 * pi * H0^3 / ((tand(alpha))^2);
flaggy2 = 0;
for i = 2:length(t)
    dVdt = P4 * pi * R^2 + RC * P4 * (Ab - pi * R^2) - E * pi * R^2;
    V(i) = V(i-1) + dVdt * dt;
    V(i) = max(0, V(i));
    
    Hc2(i) = ((3 * V(i) * (tand(alpha))^2) / pi)^(1/3);
    R = Hc2(i) / tand(alpha);
    
    if (Hc2(i) < (.01 * (zF - zL1) + zL1 - z0) && flaggy2 == 0)
        flaggy2 = i;
    end
    
end    

t_99_drain = t(flaggy2);
disp('2c. The basin will drain 99% of the difference between current and');% paleo');
disp(['paleo levels in ' num2str(t_99_drain, '%.0f') ' years.']);
disp(' ');
disp('This indicates moderate hysteresis, though more sophisticated');
disp('analysis would be helpful.');
disp(' ');



%%%%%% part d
%for simplicity, will keep the same P, E, RC, Ab, alpha from 2b.

P = 0.1;    %m/yr
RC = 0.5;
E = 2.5;    %m/yr

Ab = 1000 * 1000^2; %m2
alpha = 4;

dt = .1;
tend = 200;
t = 0:dt:tend;

H0 = 0;
H = zeros(size(t));
H(1) = H0;

R = H0 / tand(alpha);
Al = zeros(size(t));
Al(1) = pi * R^2;
V = 1/3 * pi * H0^3 / ((tand(alpha))^2);
% stardiff = (1-exp(-1)) * (zF - zL1);
% diff(1) = 999;

for i = 2:length(t)
    dVdt = P * pi * R^2 + RC * P * (Ab - pi * R^2) - E * pi * R^2;
    V(i) = V(i-1) + dVdt * dt;
    V(i) = max(0, V(i));
    
    H(i) = ((3 * V(i) * (tand(alpha))^2) / pi) ^ (1/3);
%     zL3(i) = z0 + H(i);
    
    R = H(i) / tand(alpha);
    Al(i) = pi*R^2;
    
%     diff(i) = (H(i) - H0);
    
    
end

% zL3 = z0 + H;
% [mint star] = min(abs(diff-stardiff));

% tstar = t(star)

figure('units', 'normalized', 'position', [.5 .3 .45 .6]);
subplot(3,1,1)
plot(t, H, 'r');
% xlabel('t (yrs)', 'fontsize', 14);
ylabel('H (m)', 'fontsize', 14);
axis([0 tend 0 1.1*max(H)]);
set(gca, 'XTick', 0:tend/5:tend);

subplot(3,1,2)
plot(t, Al, 'b');
ylabel('A_L (m^2)', 'fontsize', 14);
axis([0 tend 0 1.1*max(Al)]);
set(gca, 'XTick', 0:tend/5:tend);

subplot(3,1,3)
plot(t, V, 'k');
ylabel('V (m^3)', 'fontsize', 14);
xlabel('t (yr)', 'fontsize', 14);
axis([0 tend 0 1.1*max(V)]);
set(gca, 'XTick', 0:tend/5:tend);

% determine t_star (characteristic time) based on V
Vfin = V(end);
Vstar = V(1) + (1 - exp(-1)) * (Vfin - V(1));    
diff = abs(V - Vstar);
[mindiff star] = min(diff);

% Hfin = H(end);
% Hstar = H(1) + (1 - exp(-1)) * (Hfin - H(1));
% diffH = abs(H - Hstar);
% [mindiff starH] = min(diffH);

t_star_2d = t(star);
% t_star_H = t(starH)

disp(['2d. It will take the empty basin ' num2str(t_star_2d, '%.0f')]);
disp('years to fill to (1 - 1/e) of its steady state volume.');
disp(' ');




